@@ -34,8 +34,6 @@
  * gst-launch-1.0 filesrc location=test.dv ! dvdemux name=demux ! dvdec ! xvimagesink
  * ]| This pipeline decodes and renders the raw DV stream to a videosink.
  * </refsect2>
- *
- * Last reviewed on 2006-02-28 (0.10.3)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,15 +58,15 @@
 #define PAL_FRAMERATE_NUMERATOR 25
 #define PAL_FRAMERATE_DENOMINATOR 1
 
-#define PAL_NORMAL_PAR_X        59
-#define PAL_NORMAL_PAR_Y        54
-#define PAL_WIDE_PAR_X          118
-#define PAL_WIDE_PAR_Y          81
+#define PAL_NORMAL_PAR_X        16
+#define PAL_NORMAL_PAR_Y        15
+#define PAL_WIDE_PAR_X          64
+#define PAL_WIDE_PAR_Y          45
 
-#define NTSC_NORMAL_PAR_X       10
-#define NTSC_NORMAL_PAR_Y       11
-#define NTSC_WIDE_PAR_X         40
-#define NTSC_WIDE_PAR_Y         33
+#define NTSC_NORMAL_PAR_X       8
+#define NTSC_NORMAL_PAR_Y       9
+#define NTSC_WIDE_PAR_X         32
+#define NTSC_WIDE_PAR_Y         27
 
 #define DV_DEFAULT_QUALITY DV_QUALITY_BEST
 #define DV_DEFAULT_DECODE_NTH 1
@@ -393,10 +391,9 @@ gst_dvdec_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GstCaps *caps;
 
       gst_event_parse_caps (event, &caps);
-      gst_dvdec_sink_setcaps (dvdec, caps);
+      res = gst_dvdec_sink_setcaps (dvdec, caps);
       gst_event_unref (event);
       event = NULL;
-      res = TRUE;
       break;
     }
 
@@ -422,7 +419,7 @@ gst_dvdec_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
   GstBuffer *outbuf;
   GstFlowReturn ret = GST_FLOW_OK;
   guint length;
-  guint64 cstart, cstop;
+  guint64 cstart = GST_CLOCK_TIME_NONE, cstop = GST_CLOCK_TIME_NONE;
   gboolean PAL, wide;
 
   dvdec = GST_DVDEC (parent);
@@ -482,6 +479,9 @@ gst_dvdec_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     GstCaps *caps;
 
     caps = gst_pad_get_current_caps (dvdec->srcpad);
+    if (!caps)
+      goto not_negotiated;
+
     gst_dvdec_negotiate_pool (dvdec, caps, &dvdec->vinfo);
     gst_caps_unref (caps);
   }
@@ -519,8 +519,15 @@ gst_dvdec_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
   GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buf);
   GST_BUFFER_OFFSET_END (outbuf) = GST_BUFFER_OFFSET_END (buf);
-  GST_BUFFER_TIMESTAMP (outbuf) = cstart;
-  GST_BUFFER_DURATION (outbuf) = cstop - cstart;
+
+  /* FIXME : Compute values when using non-TIME segments,
+   * but for the moment make sure we at least don't set bogus values
+   */
+  if (GST_CLOCK_TIME_IS_VALID (cstart)) {
+    GST_BUFFER_TIMESTAMP (outbuf) = cstart;
+    if (GST_CLOCK_TIME_IS_VALID (cstop))
+      GST_BUFFER_DURATION (outbuf) = cstop - cstart;
+  }
 
   ret = gst_pad_push (dvdec->srcpad, outbuf);
 
