@@ -29,11 +29,26 @@
 GST_DEBUG_CATEGORY_STATIC (gst_jack_audio_client_debug);
 #define GST_CAT_DEFAULT gst_jack_audio_client_debug
 
+static void
+jack_log_error (const gchar * msg)
+{
+  GST_ERROR ("%s", msg);
+}
+
+static void
+jack_info_error (const gchar * msg)
+{
+  GST_INFO ("%s", msg);
+}
+
 void
 gst_jack_audio_client_init (void)
 {
   GST_DEBUG_CATEGORY_INIT (gst_jack_audio_client_debug, "jackclient", 0,
       "jackclient helpers");
+
+  jack_set_error_function (jack_log_error);
+  jack_set_info_function (jack_info_error);
 }
 
 /* a list of global connections indexed by id and server. */
@@ -70,6 +85,7 @@ struct _GstJackAudioClient
   GstJackClientType type;
   gboolean active;
   gboolean deactivate;
+  gboolean server_down;
 
   JackShutdownCallback shutdown;
   JackProcessCallback process;
@@ -212,12 +228,16 @@ jack_shutdown_cb (void *arg)
   for (walk = conn->src_clients; walk; walk = g_list_next (walk)) {
     GstJackAudioClient *client = (GstJackAudioClient *) walk->data;
 
+    client->server_down = TRUE;
+    g_cond_signal (&conn->flush_cond);
     if (client->shutdown)
       client->shutdown (client->user_data);
   }
   for (walk = conn->sink_clients; walk; walk = g_list_next (walk)) {
     GstJackAudioClient *client = (GstJackAudioClient *) walk->data;
 
+    client->server_down = TRUE;
+    g_cond_signal (&conn->flush_cond);
     if (client->shutdown)
       client->shutdown (client->user_data);
   }
@@ -505,6 +525,7 @@ gst_jack_audio_client_new (const gchar * id, const gchar * server,
   client->buffer_size = buffer_size;
   client->sample_rate = sample_rate;
   client->user_data = user_data;
+  client->server_down = FALSE;
 
   /* add the client to the connection */
   gst_jack_audio_connection_add_client (conn, client);
@@ -585,7 +606,7 @@ gst_jack_audio_client_set_active (GstJackAudioClient * client, gboolean active)
     client->deactivate = TRUE;
 
     /* need to wait for process_cb run once more */
-    while (client->deactivate)
+    while (client->deactivate && !client->server_down)
       g_cond_wait (&client->conn->flush_cond, &client->conn->lock);
   }
   client->active = active;
