@@ -644,6 +644,9 @@ refuse_caps:
   }
 }
 
+static void gst_avi_mux_audsink_set_fields (GstAviMux * avimux,
+    GstAviAudioPad * avipad);
+
 static GstFlowReturn
 gst_avi_mux_audsink_scan_mpeg_audio (GstAviMux * avimux, GstAviPad * avipad,
     GstBuffer * buffer)
@@ -688,9 +691,10 @@ gst_avi_mux_audsink_scan_mpeg_audio (GstAviMux * avimux, GstAviPad * avipad,
     spf = 576;
   }
 
-  if (G_UNLIKELY (avipad->hdr.scale <= 1))
+  if (G_UNLIKELY (avipad->hdr.scale <= 1)) {
     avipad->hdr.scale = spf;
-  else if (G_UNLIKELY (avipad->hdr.scale != spf)) {
+    gst_avi_mux_audsink_set_fields (avimux, (GstAviAudioPad *) avipad);
+  } else if (G_UNLIKELY (avipad->hdr.scale != spf)) {
     GST_WARNING_OBJECT (avimux, "input mpeg audio has varying frame size");
     goto cbr_fallback;
   }
@@ -709,6 +713,7 @@ cbr_fallback:
   {
     GST_WARNING_OBJECT (avimux, "falling back to CBR muxing");
     avipad->hdr.scale = 1;
+    gst_avi_mux_audsink_set_fields (avimux, (GstAviAudioPad *) avipad);
     /* no need to check further */
     avipad->hook = NULL;
     goto done;
@@ -1780,6 +1785,10 @@ gst_avi_mux_stop_file (GstAviMux * avimux)
   GSList *node;
   GstSegment segment;
 
+  /* Do not write index and header, if the index has no data */
+  if (avimux->idx == NULL)
+    return GST_FLOW_OK;
+
   /* if bigfile, rewrite header, else write indexes */
   /* don't bail out at once if error, still try to re-write header */
   if (avimux->video_pads > 0) {
@@ -1826,7 +1835,9 @@ gst_avi_mux_stop_file (GstAviMux * avimux)
         audpad->auds.blockalign = audpad->max_audio_chunk;
       if (audpad->auds.blockalign == 0)
         audpad->auds.blockalign = 1;
-      gst_avi_mux_audsink_set_fields (avimux, audpad);
+      /* note that hdr.rate is actually used by demux in cbr case */
+      if (avipad->hdr.scale <= 1)
+        avipad->hdr.rate = audpad->auds.av_bps / audpad->auds.blockalign;
       avimux->avi_hdr.max_bps += audpad->auds.av_bps;
       avipad->hdr.length = gst_util_uint64_scale (audpad->audio_time,
           avipad->hdr.rate, avipad->hdr.scale * GST_SECOND);
