@@ -148,7 +148,7 @@ static const GEnumValue methods_types[] = {
       "greedyh"},
   {GST_DEINTERLACE_GREEDY_L, "Motion Adaptive: Simple Detection", "greedyl"},
   {GST_DEINTERLACE_VFIR, "Blur Vertical", "vfir"},
-  {GST_DEINTERLACE_LINEAR, "Television: Full resolution", "linear"},
+  {GST_DEINTERLACE_LINEAR, "Linear", "linear"},
   {GST_DEINTERLACE_LINEAR_BLEND, "Blur: Temporal (Do Not Use)",
       "linearblend"},
   {GST_DEINTERLACE_SCALER_BOB, "Double lines", "scalerbob"},
@@ -2508,9 +2508,20 @@ gst_deinterlace_setcaps (GstDeinterlace * self, GstPad * pad, GstCaps * caps)
   GstCaps *srccaps = NULL;
   GstVideoInterlaceMode interlacing_mode;
   gint fps_n, fps_d;
+  GstCaps *current_caps;
 
   gst_pad_check_reconfigure (self->srcpad);
 
+  if ((current_caps = gst_pad_get_current_caps (pad))) {
+    if (gst_caps_is_equal (caps, current_caps)) {
+      GST_DEBUG_OBJECT (pad, "Got same caps again, returning");
+      gst_caps_unref (current_caps);
+      return TRUE;
+    }
+    gst_deinterlace_reset_history (self, FALSE);
+    gst_caps_unref (current_caps);
+  }
+
   if (self->locking != GST_DEINTERLACE_LOCKING_NONE) {
     if (self->low_latency == -1)
       self->low_latency = gst_deinterlace_get_latency (self);
