@@ -22,6 +22,7 @@
 
 #include <gst/gst.h>
 #include <gst/base/gstadapter.h>
+#include <gst/base/gstflowcombiner.h>
 #include "gstindex.h"
 
 G_BEGIN_DECLS
@@ -47,6 +48,7 @@ typedef enum
   FLV_STATE_TAG_SCRIPT,
   FLV_STATE_SEEK,
   FLV_STATE_DONE,
+  FLV_STATE_SKIP,
   FLV_STATE_NONE
 } GstFlvDemuxState;
 
@@ -73,6 +75,8 @@ struct _GstFlvDemux
 
   GstAdapter *adapter;
 
+  GstFlowCombiner *flowcombiner;
+
   GstSegment segment;
 
   GstEvent *new_seg_event;
@@ -114,7 +118,7 @@ struct _GstFlvDemux
   gboolean got_par;
   GstBuffer * video_codec_data;
   GstClockTime video_start;
-  guint32 last_video_pts;
+  guint32 last_video_dts;
   GstClockTime video_time_offset;
   gdouble framerate;
 
@@ -128,6 +132,11 @@ struct _GstFlvDemux
 
   gboolean no_more_pads;
 
+#ifndef GST_DISABLE_DEBUG
+  gboolean no_audio_warned;
+  gboolean no_video_warned;
+#endif
+
   gboolean seeking;
   gboolean building_index;
   gboolean indexed; /* TRUE if index is completely built */
