@@ -55,6 +55,8 @@ typedef struct _GstMatroskaMetaSeekIndex {
 
 typedef gboolean (*GstMatroskaCapsFunc) (GstPad *pad, GstCaps *caps);
 
+typedef struct _GstMatroskaMux GstMatroskaMux;
+
 /* all information needed for one matroska stream */
 typedef struct
 {
@@ -62,7 +64,10 @@ typedef struct
   GstMatroskaCapsFunc capsfunc;
   GstMatroskaTrackContext *track;
 
-  guint64 duration;
+  GstMatroskaMux *mux;
+
+  GstTagList *tags;
+
   GstClockTime start_ts;
   GstClockTime end_ts;    /* last timestamp + (if available) duration */
   guint64 default_duration_scaled;
@@ -70,7 +75,7 @@ typedef struct
 GstMatroskaPad;
 
 
-typedef struct _GstMatroskaMux {
+struct _GstMatroskaMux {
   GstElement     element;
   
   /* < private > */
@@ -99,7 +104,6 @@ typedef struct _GstMatroskaMux {
   GstMatroskaIndex *index;
   guint          num_indexes;
   GstClockTimeDiff min_index_interval;
-  gboolean       streamable;
  
   /* timescale in the file */
   guint64        time_scale;
@@ -135,7 +139,7 @@ typedef struct _GstMatroskaMux {
 
   /* used uids */
   GArray *used_uids;
-} GstMatroskaMux;
+};
 
 typedef struct _GstMatroskaMuxClass {
   GstElementClass parent;
