@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 /**
  * SECTION:element-splitfilesrc
@@ -30,17 +30,14 @@
  * directories). The results will be sorted.
  *
  * <refsect2>
- * <title>Example launch line</title>
+ * <title>Example launch lines</title>
  * |[
- * gst-launch splitfilesrc location="/path/to/part-*.mpg" ! decodebin ! ... \
+ * gst-launch-1.0 splitfilesrc location="/path/to/part-*.mpg" ! decodebin ! ...
  * ]| Plays the different parts as if they were one single MPEG file.
+ * |[
+ * gst-launch-1.0 playbin uri="splitfile://path/to/foo.avi.*"
+ * ]| Plays the different parts as if they were one single AVI file.
  * </refsect2>
- *
- * Since: 0.10.31
- */
-
-/* TODO:
- *  - implement splitfile:// URI handler?
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,16 +45,10 @@
 #endif
 
 #include "gstsplitfilesrc.h"
-#include "patternspec.h"
+#include "gstsplitutils.h"
 
 #include <string.h>
 
-#ifdef G_OS_WIN32
-#define DEFAULT_PATTERN_MATCH_MODE MATCH_MODE_UTF8
-#else
-#define DEFAULT_PATTERN_MATCH_MODE MATCH_MODE_AUTO
-#endif
-
 enum
 {
   PROP_LOCATION = 1
@@ -65,6 +56,8 @@ enum
 
 #define DEFAULT_LOCATION NULL
 
+static void gst_split_file_src_uri_handler_init (gpointer g_iface,
+    gpointer iface_data);
 static void gst_split_file_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_split_file_src_get_property (GObject * object, guint prop_id,
@@ -88,7 +81,10 @@ GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (splitfilesrc_debug);
 #define GST_CAT_DEFAULT splitfilesrc_debug
 
-G_DEFINE_TYPE (GstSplitFileSrc, gst_split_file_src, GST_TYPE_BASE_SRC);
+
+G_DEFINE_TYPE_WITH_CODE (GstSplitFileSrc, gst_split_file_src, GST_TYPE_BASE_SRC,
+    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
+        gst_split_file_src_uri_handler_init));
 
 #ifdef G_OS_WIN32
 #define WIN32_BLURB " Location string must be in UTF-8 encoding (on Windows)."
@@ -182,6 +178,25 @@ gst_split_file_src_get_size (GstBaseSrc * basesrc, guint64 * size)
   return TRUE;
 }
 
+static void
+gst_split_file_src_set_location (GstSplitFileSrc * src, const char *location)
+{
+  GST_OBJECT_LOCK (src);
+  g_free (src->location);
+
+  if (location != NULL && g_str_has_prefix (location, "splitfile://"))
+    src->location = gst_uri_get_location (location);
+  else
+    src->location = g_strdup (location);
+#ifdef G_OS_WIN32
+  if (!g_utf8_validate (src->location, -1, NULL)) {
+    g_warning ("splitfilesrc 'location' property must be in UTF-8 "
+        "encoding on Windows");
+  }
+#endif
+  GST_OBJECT_UNLOCK (src);
+}
+
 static void
 gst_split_file_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -190,16 +205,7 @@ gst_split_file_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_LOCATION:
-      GST_OBJECT_LOCK (src);
-      g_free (src->location);
-      src->location = g_value_dup_string (value);
-#ifdef G_OS_WIN32
-      if (!g_utf8_validate (src->location, -1, NULL)) {
-        g_warning ("splitfilesrc 'location' property must be in UTF-8 "
-            "encoding on Windows");
-      }
-#endif
-      GST_OBJECT_UNLOCK (src);
+      gst_split_file_src_set_location (src, g_value_get_string (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -225,84 +231,6 @@ gst_split_file_src_get_property (GObject * object, guint prop_id,
   }
 }
 
-static int
-gst_split_file_src_array_sortfunc (gchar ** a, gchar ** b)
-{
-  return strcmp (*a, *b);
-}
-
-static gchar **
-gst_split_file_src_find_files (GstSplitFileSrc * src, const gchar * dirname,
-    const gchar * basename, GError ** err)
-{
-  PatternSpec *pspec;
-  GPtrArray *files;
-  const gchar *name;
-  GDir *dir;
-
-  if (dirname == NULL || basename == NULL)
-    goto invalid_location;
-
-  GST_INFO_OBJECT (src, "checking in directory '%s' for pattern '%s'",
-      dirname, basename);
-
-  dir = g_dir_open (dirname, 0, err);
-  if (dir == NULL)
-    return NULL;
-
-  if (DEFAULT_PATTERN_MATCH_MODE == MATCH_MODE_UTF8 &&
-      !g_utf8_validate (basename, -1, NULL)) {
-    goto not_utf8;
-  }
-
-  /* mode will be AUTO on linux/unix and UTF8 on win32 */
-  pspec = pattern_spec_new (basename, DEFAULT_PATTERN_MATCH_MODE);
-
-  files = g_ptr_array_new ();
-
-  while ((name = g_dir_read_name (dir))) {
-    GST_TRACE_OBJECT (src, "check: %s", name);
-    if (pattern_match_string (pspec, name)) {
-      GST_DEBUG_OBJECT (src, "match: %s", name);
-      g_ptr_array_add (files, g_build_filename (dirname, name, NULL));
-    }
-  }
-
-  if (files->len == 0)
-    goto no_matches;
-
-  g_ptr_array_sort (files, (GCompareFunc) gst_split_file_src_array_sortfunc);
-  g_ptr_array_add (files, NULL);
-
-  pattern_spec_free (pspec);
-  g_dir_close (dir);
-
-  return (gchar **) g_ptr_array_free (files, FALSE);
-
-/* ERRORS */
-invalid_location:
-  {
-    g_set_error_literal (err, G_FILE_ERROR, G_FILE_ERROR_INVAL,
-        "No filename specified.");
-    return NULL;
-  }
-not_utf8:
-  {
-    g_dir_close (dir);
-    g_set_error_literal (err, G_FILE_ERROR, G_FILE_ERROR_INVAL,
-        "Filename pattern must be UTF-8 on Windows.");
-    return NULL;
-  }
-no_matches:
-  {
-    pattern_spec_free (pspec);
-    g_dir_close (dir);
-    g_set_error_literal (err, G_FILE_ERROR, G_FILE_ERROR_NOENT,
-        "Found no files matching the pattern.");
-    return NULL;
-  }
-}
-
 static gboolean
 gst_split_file_src_start (GstBaseSrc * basesrc)
 {
@@ -323,7 +251,7 @@ gst_split_file_src_start (GstBaseSrc * basesrc)
   }
   GST_OBJECT_UNLOCK (src);
 
-  files = gst_split_file_src_find_files (src, dirname, basename, &err);
+  files = gst_split_util_find_files (dirname, basename, &err);
 
   if (files == NULL || *files == NULL)
     goto no_files;
@@ -440,24 +368,37 @@ gst_split_file_src_stop (GstBaseSrc * basesrc)
   return TRUE;
 }
 
+static gint
+gst_split_file_src_part_search (GstFilePart * part, guint64 * offset,
+    gpointer user_data)
+{
+  if (*offset > part->stop)
+    return -1;                  /* The target is after this part */
+  else if (*offset < part->start)
+    return 1;                   /* The target is before this part */
+  else
+    return 0;                   /* This is the target part */
+}
+
 static gboolean
 gst_split_file_src_find_part_for_offset (GstSplitFileSrc * src, guint64 offset,
     guint * part_number)
 {
+  gboolean res = TRUE;
   GstFilePart *part;
-  guint i;
 
-  /* TODO: could use gst_util_array_binary_search() here */
-  part = src->parts;
-  for (i = 0; i < src->num_parts; ++i) {
-    if (offset >= part->start && offset <= part->stop) {
-      *part_number = i;
-      return TRUE;
-    }
-    ++part;
-  }
+  part =
+      gst_util_array_binary_search (src->parts, src->num_parts,
+      sizeof (GstFilePart),
+      (GCompareDataFunc) gst_split_file_src_part_search,
+      GST_SEARCH_MODE_AFTER, &offset, NULL);
 
-  return FALSE;
+  if (part)
+    *part_number = part - src->parts;
+  else
+    res = FALSE;
+
+  return res;
 }
 
 static GstFlowReturn
@@ -603,3 +544,55 @@ cancelled:
     return GST_FLOW_FLUSHING;
   }
 }
+
+static guint
+gst_split_file_src_uri_get_type (GType type)
+{
+  return GST_URI_SRC;
+}
+
+static const gchar *const *
+gst_split_file_src_uri_get_protocols (GType type)
+{
+  static const gchar *protocols[] = { "splitfile", NULL };
+
+  return (const gchar * const *) protocols;
+}
+
+static gchar *
+gst_split_file_src_uri_get_uri (GstURIHandler * handler)
+{
+  GstSplitFileSrc *src = GST_SPLIT_FILE_SRC (handler);
+  gchar *ret;
+
+  GST_OBJECT_LOCK (src);
+  if (src->location != NULL)
+    ret = g_strdup_printf ("splitfile://%s", src->location);
+  else
+    ret = NULL;
+  GST_OBJECT_UNLOCK (src);
+
+  return ret;
+}
+
+static gboolean
+gst_split_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri,
+    GError ** error)
+{
+  GstSplitFileSrc *src = GST_SPLIT_FILE_SRC (handler);
+
+  gst_split_file_src_set_location (src, uri);
+
+  return TRUE;
+}
+
+static void
+gst_split_file_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
+{
+  GstURIHandlerInterface *iface = (GstURIHandlerInterface *) g_iface;
+
+  iface->get_type = gst_split_file_src_uri_get_type;
+  iface->get_protocols = gst_split_file_src_uri_get_protocols;
+  iface->get_uri = gst_split_file_src_uri_get_uri;
+  iface->set_uri = gst_split_file_src_uri_set_uri;
+}
