@@ -94,8 +94,7 @@ enum
   PROP_BORDER,
   PROP_DEPTH,
   PROP_POSITION,
-  PROP_INVERT,
-  PROP_LAST,
+  PROP_INVERT
 };
 
 #define AYUV_SIZE(w,h)     ((w) * (h) * 4)
@@ -191,8 +190,6 @@ gst_smpte_alpha_class_init (GstSMPTEAlphaClass * klass)
    * GstSMPTEAlpha:invert:
    *
    * Set to TRUE to invert the transition mask (ie. flip it horizontally).
-   *
-   * Since: 0.10.23
    */
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_INVERT,
       g_param_spec_boolean ("invert", "Invert",
@@ -618,22 +615,20 @@ gst_smpte_alpha_set_info (GstVideoFilter * vfilter, GstCaps * incaps,
 {
   GstSMPTEAlpha *smpte = GST_SMPTE_ALPHA (vfilter);
   gboolean ret;
-  gint width, height;
 
   smpte->process = NULL;
-
   smpte->in_format = GST_VIDEO_INFO_FORMAT (in_info);
   smpte->out_format = GST_VIDEO_INFO_FORMAT (out_info);
-  smpte->width = width = GST_VIDEO_INFO_WIDTH (out_info);
-  smpte->height = height = GST_VIDEO_INFO_HEIGHT (out_info);
 
   /* try to update the mask now, this will also adjust the width/height on
    * success */
   GST_OBJECT_LOCK (smpte);
   ret =
       gst_smpte_alpha_update_mask (smpte, smpte->type, smpte->invert,
-      smpte->depth, width, height);
+      smpte->depth, GST_VIDEO_INFO_WIDTH (out_info),
+      GST_VIDEO_INFO_HEIGHT (out_info));
   GST_OBJECT_UNLOCK (smpte);
+
   if (!ret)
     goto mask_failed;
 
